# -*- coding: utf-8 -*-
############################## TERMS OF USE #####################################
# The following code is provided for demonstration purpose only, and should not #
# be used without independent verification. Recorded Future makes no            #
# representations or warranties, express, implied, statutory, or otherwise,     #
# regarding any aspect of this code or of the information it may retrieve, and  #
# provides it both strictly “as-is” and without assuming responsibility for any #
# information it may retrieve. Recorded Future shall not be liable for, and you #
# assume all risk of using, the foregoing. By using this code, Customer         #
# represents that it is solely responsible for having all necessary licenses,   #
# permissions, rights, and/or consents to connect to third party APIs, and that #
# it is solely responsible for having all necessary licenses, permissions,      #
# rights, and/or consents to any data accessed from any third party API.        #
#################################################################################

import json
from post import buildCollectiveInsightsPayload, submitCollectiveInsights
from playbook_app import PlaybookApp

# pylint: disable=attribute-defined-outside-init
class App(PlaybookApp):
    """Playbook App"""

    def __init__(self, _tcex):
        """Initialize class properties."""

        super().__init__(_tcex)

        # Initialize Collective Insights API URL and app output parameters
        self.url = "https://api.recordedfuture.com/collective-insights/detections"
        self.status = ""
        self.summary = ""
        self.raw = ""
        self.has_incident = False

    def run(self):
        """Run the App main logic."""

        # read inputs
        self.tcex.log.info("Parsing playbook parameters.")
        self.token = self.tcex.playbook.read(self.args.token)
        
        self.incident = self.tcex.playbook.read(self.args.incident)
        self.indicators = self.tcex.playbook.read(self.args.indicators)
        self.groups = self.tcex.playbook.read(self.args.groups)
        
        advanced_params = parseAdvancedParameters(self.tcex.playbook.read(self.args.advanced))
        
        if self.incident:
            self.has_incident = True
            self.incident_name = self.incident["value"]
            owner = self.incident["ownerName"]
            batch = self.tcex.batch(owner=owner)
            self.uuid = batch.generate_xid([owner, self.incident_name])


        self.tcex.log.info("Building Collective Insights payload.")
        collective_insights_payload = buildCollectiveInsightsPayload(self, advanced_params)

        self.tcex.log.info("Sending data to Collective Insights API.")
        response_json = submitCollectiveInsights(self, collective_insights_payload) # json

        if "result" in response_json:
            self.tcex.log.info("Successfully sent IOCs to Collective Insights API.")
            self.tcex.log.info("Building playbook output variables.")
            self.raw = json.dumps(response_json)
            self.status = json.dumps(response_json["result"]["status"])
            self.summary = json.dumps(response_json["result"]["summary"])
        else:
            error_message = response_json["message"]
            log_message = "Unable to send IOC to Collective Insights: " + error_message
            self.tcex.log.error(log_message)
            self.tcex.exit(1, log_message)

        # set the App exit message
        self.exit_message = 'IOCs sent to Recorded Future Collective Insights.'


    def write_output(self):
        """Write the Playbook output variables."""

        self.tcex.log.info('Writing Output')
        self.tcex.playbook.create_output('status', self.status)
        self.tcex.playbook.create_output('summary', self.summary)
        self.tcex.playbook.create_output('raw', self.raw)


# Parsing arguments helper function
def parseAdvancedParameters(advanced) -> dict:
    """
    Takes an OrderedDict object and turns it into a regular dictionary.
    """

    result = {}
    for item in advanced:
        result[item['key']] = item['value']
    return result